/*
VideoCallActivity.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.linphone;



import org.linphone.core.LinphoneCore;
import org.linphone.core.Log;
import org.linphone.core.Version;
import org.linphone.core.VideoSize;
import org.linphone.core.video.AndroidCameraRecordManager;
import org.linphone.core.video.AndroidCameraRecordManager.OnCapturingStateChangedListener;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.EditText;
import android.widget.TextView;

/**
 * For Android SDK >= 5
 * @author Guillaume Beraudo
 *
 * modifed by Martin Hrlin to also support textcalls
 */
public class TextCallActivity extends SoftVolumeActivity implements OnCapturingStateChangedListener, LinphoneManager.RTTRecievedListener {
	private SurfaceView mVideoView;
	private SurfaceView mVideoCaptureView;
	private AndroidCameraRecordManager recordManager;
	public static boolean launched = false;
	private WakeLock mWakeLock;
	private static final int capturePreviewLargestDimension = 150;
	private Handler handler = new Handler();

	private EditText mTextDisplay;
	private EditText mTextInput;
	private TextView mTextInfo;
	private RttTextWatcher mRttTextWatcher;
	private boolean mHasVideo;
	private boolean mShowHint=true;
	public void onCreate(Bundle savedInstanceState) {
		launched = true;
		mHasVideo=LinphoneManager.getLc().isVideoEnabled();
		
		Log.d("onCreate VideoCallActivity");
		super.onCreate(savedInstanceState);
		if (mHasVideo){
			setContentView(R.layout.textvideocall);
		} else {
			setContentView(R.layout.textcall);
		}

		if (mHasVideo){
			mVideoView = (SurfaceView) findViewById(R.id.video_surface); 
			LinphoneCore lc = LinphoneManager.getLc();
			lc.setVideoWindow(mVideoView);
	
			mVideoCaptureView = (SurfaceView) findViewById(R.id.video_capture_surface);
	
			recordManager = AndroidCameraRecordManager.getInstance();
			recordManager.setOnCapturingStateChanged(this);
			recordManager.setSurfaceView(mVideoCaptureView);
			mVideoCaptureView.setZOrderOnTop(true);
		}

		mTextInput=(EditText)findViewById(R.id.text_input);
		mRttTextWatcher=new RttTextWatcher();
		mTextInput.addTextChangedListener(mRttTextWatcher);
		mTextInput.setOnTouchListener(new OnTouchListener() {

			@Override
			public boolean onTouch(View arg0, MotionEvent arg1) {
				mShowHint=false;
				mTextInfo.post(new Runnable() {
					public void run() {
						mTextInfo.setVisibility(View.GONE);
					}
				});
				mTextInput.setOnTouchListener(null);
				return false;
			}
			
		});
		mTextDisplay=(EditText)findViewById(R.id.text_display);
		mTextInfo=(TextView)findViewById(R.id.text_info);
		
		if (mHasVideo){
			if (!recordManager.isMuted()) LinphoneManager.getInstance().sendStaticImage(false);
		}
		PowerManager pm = (PowerManager)getSystemService(Context.POWER_SERVICE);
		mWakeLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK|PowerManager.ON_AFTER_RELEASE,Log.TAG);
		mWakeLock.acquire();

		if (mHasVideo){
			fixScreenOrientationForOldDevices();
		}
	}

	private void fixScreenOrientationForOldDevices() {
		if (Version.sdkAboveOrEqual(Version.API08_FROYO_22)) return;

		// Force to display in portrait orientation for old devices
		// as they do not support surfaceView rotation
		setRequestedOrientation(recordManager.isCameraMountedPortrait() ?
				ActivityInfo.SCREEN_ORIENTATION_PORTRAIT :
				ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		resizeCapturePreview(mVideoCaptureView);
	}

	@Override
	protected void onResume() {
		if (mHasVideo){
			if (Version.sdkAboveOrEqual(8) && recordManager.isOutputOrientationMismatch(LinphoneManager.getLc())) {
				Log.i("Phone orientation has changed: updating call.");
				CallManager.getInstance().updateCall();
				// resizeCapturePreview by callback when recording started
			}
		}
		LinphoneManager.getInstance().setOnRTTRecievedListener(this);
		OnRTTRecieved(LinphoneManager.getLc());
		
		if (mShowHint){
			mTextInfo.postDelayed(new Runnable() {
				public void run() {
					mTextInfo.setVisibility(View.GONE);
				}
			}, 10000);
			mShowHint=false;
		} else {
			mTextInfo.setVisibility(View.GONE);
		}
		
		super.onResume();
	}


	private void rewriteToggleCameraItem(MenuItem item) {
		if (recordManager.isRecording()) {
			item.setTitle(getString(R.string.menu_videocall_toggle_camera_disable));
		} else {
			item.setTitle(getString(R.string.menu_videocall_toggle_camera_enable));
		}
	}


	private void rewriteChangeResolutionItem(MenuItem item) {
		if (BandwidthManager.getInstance().isUserRestriction()) {
			item.setTitle(getString(R.string.menu_videocall_change_resolution_when_low_resolution));
		} else {
			item.setTitle(getString(R.string.menu_videocall_change_resolution_when_high_resolution));
		}
	}


	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the currently selected menu XML resource.
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.videocall_activity_menu, menu);
		menu.add(getString(R.string.menu_textcall_clear_text));
		if (mHasVideo){
			rewriteToggleCameraItem(menu.findItem(R.id.videocall_menu_toggle_camera));
			rewriteChangeResolutionItem(menu.findItem(R.id.videocall_menu_change_resolution));
			if (!recordManager.hasSeveralCameras()) {
				menu.findItem(R.id.videocall_menu_switch_camera).setVisible(false);
			}
		} else {
			(menu.findItem(R.id.videocall_menu_toggle_camera)).setVisible(false);
			(menu.findItem(R.id.videocall_menu_switch_camera)).setVisible(false);
			(menu.findItem(R.id.videocall_menu_change_resolution)).setVisible(false);
		}
		
		return true;
	}



	/**
	 * Base capture frame on streamed dimensions and orientation.
	 * @param sv capture surface view to resize the layout
	 * @param vs video size from which to calculate the dimensions
	 */
	private void resizeCapturePreview(SurfaceView sv) {
		LayoutParams lp = sv.getLayoutParams();
		VideoSize vs = LinphoneManager.getLc().getPreferredVideoSize();

		float newRatio = (float) vs.width / vs.height;

		if (vs.isPortrait()) {
			lp.height = capturePreviewLargestDimension;
			lp.width = Math.round(lp.height * newRatio);
		} else {
			lp.width = capturePreviewLargestDimension;
			lp.height = Math.round(lp.width / newRatio);
		}

		sv.setLayoutParams(lp);
	}



	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.videocall_menu_back_to_dialer:
			finish();
			break;
		case R.id.videocall_menu_change_resolution:
			LinphoneManager.getInstance().changeResolution();
			rewriteChangeResolutionItem(item);
			break;
		case R.id.videocall_menu_terminate_call:
			LinphoneManager.getInstance().terminateCall();
			break;
		case R.id.videocall_menu_toggle_camera:
			LinphoneManager.getInstance().toggleCameraMuting();
			rewriteToggleCameraItem(item);
			break;
		case R.id.videocall_menu_switch_camera:
			LinphoneManager.getInstance().switchCamera();
			fixScreenOrientationForOldDevices();
			break;
		default:
			if (item.getTitle().equals(getString(R.string.menu_textcall_clear_text))){
				mTextDisplay.post(new Runnable() {
					public void run() {
						mTextDisplay.setText("");
					}
				});
				mTextInput.post(new Runnable() {
					public void run() {
						mRttTextWatcher.setEnable(false);
						//mTextInput.removeTextChangedListener(mTextWatcher);
						mTextInput.setText("");
						mRttTextWatcher.setEnable(true);
						//mTextInput.addTextChangedListener(mTextWatcher);
					}
				});
				return true;
			}
			Log.e("Unknown menu item [",item,"]");
			break;
		}

		return true;
	}


	@Override
	protected void onDestroy() {
		LinphoneManager.getInstance().setOnRTTRecievedListener(null);
		launched = false;
		super.onDestroy();
	}

	@Override
	protected void onPause() {
		Log.d("onPause VideoCallActivity");
		LinphoneManager.getInstance().setOnRTTRecievedListener(null);
		LinphoneManager.getInstance().sendStaticImage(true);
		if (mWakeLock.isHeld())	mWakeLock.release();
		super.onPause();
	}

	public void captureStarted() {
		handler.post(new Runnable() {
			public void run() {
				resizeCapturePreview(mVideoCaptureView);
			}
		});
	}

	public void captureStopped() {
	}

	@Override
	public void OnRTTRecieved(final LinphoneCore lc) {
		int c;
		String s="";
		int e=0;
		while ((c=lc.textStreamGetchar32())!=0){
			switch (c) {
			case 0x08:
				if (s.length()>1){
					s=s.substring(0,s.length()-1);
				} else if (s.length()==1){
					s="";
				} else {
					e++;
				}
				break;
			case 0x2028:
				s=s+'\n';
				break;
			default:
				s+=(char)c;
			}
		}
		updateTextDisplay(e,s);
	}

	
	private void updateTextDisplay(final int erase, final String newtext){
		mTextDisplay.post(new Runnable() {
			public void run() {
				String s = mTextDisplay.getText().toString();
				if (s.length()>erase){
					s=s.substring(0,s.length()-erase)+newtext;
					mTextDisplay.setText(s);
				} else {
					mTextDisplay.setText(newtext);
				}
				mTextDisplay.setSelection(mTextDisplay.getText().length());
			}
		});
	}
	
	private class RttTextWatcher implements TextWatcher {
		private CharSequence bc;
		private boolean enable;
		Handler h = new Handler();
		
		public RttTextWatcher() {
			enable = true;
			bc = null;
		}
		public void setEnable(boolean e) {
			enable = e;
		}
		
		public void afterTextChanged(Editable s) {}
		public void beforeTextChanged(CharSequence cs, int start, int count,
				int after) {
			if (!enable)
				return;
			if (count>0) {
				bc = cs.subSequence(start,start+count);
			} else {
				bc = null;
			}
		}
		
		public void onTextChanged(CharSequence cs, int start, int before,
				int count) {
			if (!enable) return;
			//LinphoneCore lc=LinphoneService.instance().getLinphoneCore();
			int ic=count;
			int remove;
			int i;
			
			if (bc != null) { 
				remove = bc.length();
			} else {
				remove = 0;
			}

			i=0;
			while (remove>0&&ic>0) {
				if (cs.charAt(start+i)!=bc.charAt(i)) {
					break;
				}
				i++;
				ic--;
				remove--;	
			}	
			
			i=count-ic;
			final CharSequence cc;
			final int rr=remove;
			
			
			if(i<count){
				cc = cs.subSequence(start+i, start+count);
			}else{
				cc = null;
			}
				
			h.post(new Runnable() {	
				@Override public void run() {
					//SEND rr BS
					LinphoneCore lc = LinphoneManager.getLc();
					for (int i=0;i<rr;i++)
						lc.textStreamPutchar32(0x08);
					//SEND CHARS IN cc
					if (cc!=null){
						for (int i=0;i<cc.length();i++)
							lc.textStreamPutchar32((int)cc.charAt(i));
					}
				}
			});
		}
	}
}
