/**
 * textstream.h
 *
 * @author Martin Hrlin
 */

/**	
 * This software was developed with funding from the National Institute 
 * on Disability and Rehabilitation Research, U.S. Department of Education, 
 * grant number H133E090001 (RERC on Telecommunications Access). However, 
 * those contents do not necessarily represent the policy of the Department 
 * of Education, and you should not assume endorsement by the Federal Government. 
 */

#ifndef _TEXT_STREAM_H
#define _TEXT_STREAM_H

typedef struct _TextStream TextStream;

/**
 * Create a new TextStream
 **/
TextStream *text_stream_new(int locport,bool_t ipv6);

/**
 * Start Stream
 **/
void text_stream_start(TextStream *ts,RtpProfile *profile,const char *rem_addr,
		       int rem_port,int rem_rtcp_port,int pt);
/**
 * Stop and destroy stream
 **/
void text_stream_stop(TextStream *ts);

/**
 * Iterate stream
 */
int text_stream_iterate(TextStream *ts);

/**
 * write a character to stream, to write an utf8 character, just call it multiple times.
 **/
char text_stream_getchar(TextStream *ts);

/**
 * returns one character from the stream, if it returns '\0' there are no more characters
 * in the stream
 **/
void text_stream_putchar(TextStream *ts,const char c);

/**
 * reads a character from the stream in UTF-32 format.
 **/
uint32_t text_stream_getchar32(TextStream *ts);

/**
 * write a character to stream in UTF-32 format.
 **/
void text_stream_putchar32(TextStream *ts,uint32_t i);

#endif
